using System;
using System.IO;
using System.Diagnostics;
using StringBuilder = System.Text.StringBuilder;
using Views;

class ViewsInstall
{
	static void Main()
	{
		new ViewsInstall().Go();
	}

	string mainspec = @"
		<Form Text='Views Installer'>
		 <vertical>
		  <Label Text='Install views to:'/>
		  <TextBox Name=InstallPath Width=450/>
		  <horizontal>
		   <Button Name=Continue/>
		   <Button Name=Cancel/>
		  </horizontal>
		 </vertical>
		</Form>";

	string searchspec = @"
		<Form Text='Views Installer'>
		 <vertical>
		  <ListBox Name=Status Width=350/>
		  <ProgressBar Name=Progress Width=350 Height=20/>
		  <horizontal>
		   <Button Name=Button1 Text='Exhaustive Search'/>
		   <button Name=Advanced Text='Manually Specify'/>
		   <Button Name=Cancel/>
		  </horizontal>
		 </vertical>
		</Form>";

	string advspec = @"
		<Form Text='Views Installer'>
		 <vertical>
		  <Label Text='Install views to:'/>
		  <TextBox Name=InstallPath Width=450/>
		  <Label Text='Path to sn.exe'/>
		  <horizontal>
		   <TextBox Name=snPath Width=450/>
		   <OpenFileDialog Name=snBrowse Text=Browse/>
		   <Button Name=snSearch Text='Exhaustive Search'/>
		  </horizontal>
		  <Label Text='Path to al.exe'/>
		  <horizontal>
		   <TextBox Name=alPath Width=450/>
		   <OpenFileDialog Name=alBrowse Text=Browse/>
		   <Button Name=alSearch Text='Exhaustive Search'/>
		  </horizontal>
		  <Label Text='Path to gacutil.exe'/>
		  <horizontal>
		   <TextBox Name=gacutilPath Width=450/>
		   <OpenFileDialog Name=gacutilBrowse Text=Browse/>
		   <Button Name=gacutilSearch Text='Exhaustive Search'/>
		  </horizontal>
		  <Label Text='Path to csc.rsp'/>
		  <horizontal>
		   <TextBox Name=cscPath Width=450/>
		   <OpenFileDialog Name=cscBrowse Text=Browse/>
		   <Button Name=cscSearch Text='Exhaustive Search'/>
		  </horizontal>
		  <horizontal>
		   <Button Name=Install/>
		   <Button Name=Cancel/>
		  </horizontal>
		 </vertical>
		</Form>";

	string successspec = @"
		<Form Text='Views Installer'>
		 <vertical>
		  <Label Text='Install successful!'/>
		  <Button Name=Close/>
		 </vertical>
		</Form>";

	string failurespec = @"
		<Form Text='Views Installer'>
		 <vertical>
		  <Label Text='Install not successful'/>
		  <ListBox Name=Log Width=300/>
		  <Button Name=Close/>
		 </vertical>
		</Form>";

	StringBuilder log;
	string[] path;
	string launchPath;
	string sn_exe;
	string al_exe;
	string gacutil_exe;
	string csc_rsp;
	string version;
	string installPath;

	void Go()
	{
		log = new StringBuilder();
		path = Environment.GetEnvironmentVariable("Path").Split(';');
		launchPath = Directory.GetCurrentDirectory();
		if (launchPath[launchPath.Length-1] != '\\')
			launchPath += "\\";

		version = GetVersion();

		Form f = new Form(mainspec);
		installPath = "C:\\Program Files\\Views" + version;
		f.PutText("InstallPath", installPath);
		string c = f.GetControl();
		if (c == null || c == "Cancel")
		{
			f.CloseGUI();
			return;
		}
		installPath = f.GetText("InstallPath");
		f.CloseGUI();

		
		f = new Form(searchspec);
		f.PutText("Status","Searching for dependancies...");

		sn_exe = SearchPath("sn.exe",path);
		al_exe = SearchPath("al.exe",path);
		gacutil_exe = SearchPath("gacutil.exe",path);
		csc_rsp = SearchPath("csc.rsp",path);

		f.PutValue("Progress",15);

		System.Collections.ArrayList probableDirs = new System.Collections.ArrayList(20);
		if (sn_exe == null || al_exe == null || gacutil_exe == null || csc_rsp == null)
		{

			if (Directory.Exists("C:\\"))
			{
				foreach (string s in Directory.GetDirectories("C:\\"))
				{
					if ((s.ToLower().IndexOf("microsoft") != -1) ||
						(s.ToLower().IndexOf(".net") != -1))
					{
						probableDirs.Add(s);
					}
				}
			}

			if (Directory.Exists("C:\\Program Files"))
			{
				foreach (string s in Directory.GetDirectories("C:\\Program Files"))
				{
					if ((s.ToLower().IndexOf("microsoft") != -1) ||
						(s.ToLower().IndexOf(".net") != -1))
					{
						probableDirs.Add(s);
					}
				}
			}

			if (Directory.Exists("C:\\Windows"))
			{
				foreach (string s in Directory.GetDirectories("C:\\Windows"))
				{
					if ((s.ToLower().IndexOf("microsoft") != -1) ||
						(s.ToLower().IndexOf(".net") != -1))
					{
						probableDirs.Add(s);
					}
				}
			}

			if (Directory.Exists("C:\\WINNT"))
			{
				foreach (string s in Directory.GetDirectories("C:\\WINNT"))
				{
					if ((s.ToLower().IndexOf("microsoft") != -1) ||
						(s.ToLower().IndexOf(".net") != -1))
					{
						probableDirs.Add(s);
					}
				}
			}

			double value = 30.0;
			double ratio = 70.0 / probableDirs.ToArray().Length;

			foreach (object o in probableDirs)
			{
				f.PutValue("Progress",(int)(value));

				if (al_exe == null)
					al_exe = Search("al.exe", (string)o );
				if (sn_exe == null)
					sn_exe = Search("sn.exe", (string)o );
				if (gacutil_exe == null)
					gacutil_exe = Search("gacutil.exe", (string)o );
				if (csc_rsp == null)
					csc_rsp = Search("csc.rsp", (string)o );

				value += ratio;
			}
		} // if (sn_exe == null || al_exe == null || gacutil_exe == null || csc_rsp == null)
		f.PutValue("Progress",100);

		bool installReady = false;
		if (sn_exe == null || al_exe == null || gacutil_exe == null || csc_rsp == null)
		{
			f.PutText("Status","Some dependancies not found in quick search."); 
			f.PutText("Status","You must manually specify them or exhaustively search.");
		}
		else 
		{
			f.PutText("Status","All dependancies were found by the quick search.");
			installReady = true;
		}

		c = null;
		for (;;)
		{
			if (installReady) f.PutText("Button1","Install");
			c = f.GetControl();
			if (c == "Button1")
			{
				if (installReady)
				{
					break;
				}
				else
				{
					f.PutValue("Progress",0);
					sn_exe = Search("sn.exe","C:\\");
					f.PutValue("Progress",25);
					gacutil_exe = Search("gacutil.exe","C:\\");
					f.PutValue("Progress",50);
					al_exe = Search("al.exe","C:\\");
					f.PutValue("Progress",75);
					csc_rsp = Search("csc.rsp","C:\\");
					f.PutValue("Progress",100);
				}

				if (sn_exe == null || al_exe == null ||
					gacutil_exe == null || csc_rsp == null)
				{
					f.PutText("Status","Some dependancies are still not specified."); 
					f.PutText("Status","You must manually specify them or exhaustively search.");
					installReady = false;
				}
				else
				{
					f.PutText("Status","All dependancies are now specified."); 
					installReady = true;
				}
			}
			if (c == "Advanced")
			{
				installReady = false;
				break;
			}
			if (c == null || c == "Cancel")
			{
				f.CloseGUI();
				return;
			}
		}
		f.CloseGUI();

		bool installSucess = false;
		if (installReady)
		{
			installSucess = Install();
		}
		else
		{
			f = new Form(advspec);
			f.PutText("InstallPath", installPath);
			f.PutText("snPath",sn_exe);
			f.PutText("alPath",al_exe);
			f.PutText("gacutilPath",gacutil_exe);
			f.PutText("cscPath",csc_rsp);
			c = f.GetControl();
			while (c != null && c != "Install" && c != "Cancel")
			{
				if (c == "snSearch")
				{
					sn_exe = Search("sn.exe","C:\\");
					f.PutText("snPath",sn_exe);
				}
				else if (c == "alSearch")
				{
					al_exe = Search("al.exe","C:\\");
					f.PutText("alPath",al_exe);
				}
				else if (c == "gacutilSearch")
				{
					gacutil_exe = Search("gacutil.exe","C:\\");
					f.PutText("gacutilPath",gacutil_exe);
				}
				else if (c == "cscSearch")
				{
					csc_rsp = Search("csc.rsp","C:\\");
					f.PutText("cscPath",csc_rsp);
				}
				else
				{
					// if it's not one of those values, it must be one of
					// the browse buttons, so we want to write to the text
					// box of similar name
					string textbox = ((new StringBuilder(c)).Replace("Browse","Path")).ToString();
					f.PutText(textbox, f.GetText(c));
				}
				c = f.GetControl();
			}

			sn_exe = f.GetText("snPath");
			al_exe = f.GetText("alPath");
			gacutil_exe = f.GetText("gacutilPath");
			csc_rsp = f.GetText("cscPath");
			installPath = f.GetText("InstallPath");

			installReady = (gacutil_exe != null && 
				gacutil_exe.IndexOf("gacutil.exe") != -1) &&
			(sn_exe != null && sn_exe.IndexOf("sn.exe") != -1) &&
			(al_exe != null && al_exe.IndexOf("al.exe") != -1) &&
			(csc_rsp != null && csc_rsp.IndexOf("csc.rsp") != -1);

			f.CloseGUI();

			if (c == null || c == "Cancel")
			{
				return;
			}
			else if (installReady)
			{
				installSucess = Install();
			}
		}

		if (installSucess)
		{
			f = new Form(successspec);
			f.GetControl();
			f.CloseGUI();
		}
		else
		{
			f = new Form(failurespec);
			string[] lines = log.ToString().Split('\n');
			for (int i = 0; i < lines.Length; i++)
			{
				f.PutText("Log",lines[i]);
			}
			f.GetControl();
			f.CloseGUI();
		}
	}

	bool Install()
	{
		if (!Directory.Exists(installPath))
		{
			try 
			{
				Directory.CreateDirectory(installPath);
			}
			catch (Exception e)
			{
				log.Append("Directory ");
				log.Append(installPath);
				log.Append(" doest not exist and could not be created\n\n");
				log.Append("Exception:\n ");
				log.Append(e.ToString());
				log.Append("\n\n");
				return false;
			}
		}

		if (Directory.Exists(installPath))
		{
			try
			{
				StreamWriter uninstaller = new StreamWriter(installPath + "\\uninstall.bat");
				uninstaller.WriteLine(String.Format("ViewsUninstall.exe gacutil=\"{0}\" version=\"{1}\" cscrsp=\"{2}\"",gacutil_exe,version,csc_rsp));
				uninstaller.WriteLine("pause");
				uninstaller.Close();

				File.Copy(launchPath + "Views.netmodule", installPath + "\\Views.netmodule", true);
				File.Copy(launchPath + "ViewsUninstall.exe", installPath + "\\ViewsUninstall.exe", true);
				Directory.SetCurrentDirectory(installPath);
			}
			catch (Exception e)
			{
				log.Append("Error writing required files to ");
				log.Append(installPath);
				log.Append("\n\n");
				log.Append("Exception:\n ");
				log.Append(e.ToString());
				log.Append("\n\n");
				Undo(1);
				return false;
			}

			try
			{
				Execute(sn_exe,"-k Views.snk");
			}
			catch (Exception e)
			{
				log.Append("Error writing strong name key.\n\n");
				log.Append("Exception:\n ");
				log.Append(e.ToString());
				log.Append("\n\n");
				Undo(2);
				return false;
			}

			try
			{
				Execute(al_exe,"/out:Views.dll /version:" + version + " /keyfile:Views.snk Views.netmodule");
			}
			catch (Exception e)
			{
				log.Append("Error linking module into assembly.\n\n");
				log.Append("Exception:\n ");
				log.Append(e.ToString());
				log.Append("\n\n");
				Undo(3);
				return false;
			}
			
			try
			{
				// remove all views version 1.x
				Execute(gacutil_exe,"/u \"Views, version=1\"");
				Execute(gacutil_exe,"/i Views.dll");
			}
			catch (Exception e)
			{
				log.Append("Error linking module into assembly.\n\n");
				log.Append("Exception:\n ");
				log.Append(e.ToString());
				log.Append("\n\n");
				Undo(4);
				return false;
			}

			bool oldViewsFound = false;

			try
			{
				StreamReader sr = new StreamReader(csc_rsp);
				string[] lines = sr.ReadToEnd().Split('\n');
				sr.Close();

				StreamWriter sw = new StreamWriter(csc_rsp, false);
				for (int j = 0; j < lines.Length; j++)
				{
					if (lines[j].ToLower().IndexOf("views.dll") == -1)
					{
						sw.WriteLine(lines[j].Trim());
					}
					else
					{
						oldViewsFound = true;
					}
				}
				string s = String.Format("/r:\"{0}\\Views.dll\"", installPath);
				sw.WriteLine(s.Trim());
				sw.Close();
			}
			catch (Exception e)
			{
				log.Append("Error inserting compiler reference.");
				log.Append("Exception:\n ");
				log.Append(e.ToString());
				log.Append("\n\n");
				return false;
			}

			try
			{
				if (oldViewsFound)
				{
					string dir = installPath.Substring(0,installPath.LastIndexOf('\\'));
					string[] dirs = Directory.GetDirectories(dir);
					foreach (string s in dirs)
					{
						if (s.ToLower().IndexOf("views") != -1)
						{
							if (s != installPath)
							{
								string[] files = Directory.GetFiles(s);
								foreach(string f in files)
								{
									File.Delete(f);
								}
								Directory.Delete(s);
								return true;
							}
						}
					}
					log.Append("And old version of views was unregistered, but the files were not deleted.");
					return false;
				}
			}
			catch (Exception)
			{
				log.Append("And old version of views was unregistered, but the files were not deleted.");
				return false;
			}

			return true;
		} // if (Directory.Exists(installPath))
		return false;
	}

	string SearchPath(string filename, string[] path)
	{
		if (filename != null)
		{
			StringBuilder fullname = null;
			for(int i = 0; i < path.Length; i++)
			{
				fullname = new StringBuilder(path[i]);

				fullname.Replace("\"","");

				if (fullname[fullname.Length-1] != '\\')
					fullname.Append("\\");

				fullname.Append(filename);

				if (File.Exists(fullname.ToString()))
					return fullname.ToString();
			}
		}
		return null;
	}

	string Search(string filename, string basepath)
	{
		if (filename != null)
		{
			StringBuilder fullname = null;

			fullname = new StringBuilder(basepath);

			fullname.Replace("\"","");

			if (fullname[fullname.Length-1] != '\\')
				fullname.Append("\\");

			fullname.Append(filename);

			if (File.Exists(fullname.ToString()))
				return fullname.ToString();

			string[] dirs = Directory.GetDirectories(basepath);

			for (int i = 0; i < dirs.Length; i++)
			{
				string s = null;
				try
				{
					s = Search(filename,dirs[i]);
				}
				catch (Exception)
				{
					continue;
				}
				if (s != null) return s;
			}
		}
		return null;
	}

	string Execute(string file, string args)
	{
		ProcessStartInfo psi = new ProcessStartInfo();
		psi.FileName = file;
		psi.Arguments = args;
		psi.RedirectStandardOutput = true;
		psi.UseShellExecute = false;

		Process p = Process.Start(psi);
		return p.StandardOutput.ReadToEnd();
	}

	string GetVersion()
	{
		string version = Views.Form.version;
		int ixs = 0;
		while (ixs < version.Length && !Char.IsWhiteSpace(version[ixs]))
			ixs++;
		while (ixs < version.Length && Char.IsWhiteSpace(version[ixs]))
			ixs++;

		int ixe = ixs;

		while ( ixe < version.Length && 
			(Char.IsDigit(version[ixe]) || version[ixe] == '.') )
			ixe++;

		return version.Substring(ixs, ixe-ixs);
	}

	void Undo(int point)
	{
		try
		{
			if (point >= 4)
			{
				Execute(gacutil_exe,"/u \"Views, version=" + version + "\"");
			}

			if (point >= 3)
			{
				File.Delete(installPath + "\\Views.dll");
			}

			if (point >= 2)
			{
				File.Delete(installPath + "\\Views.snk");
			}

			if (point >= 1)
			{
				File.Delete(installPath + "\\uninstall.bat");
				File.Delete(installPath + "\\Views.netmodule");
				File.Delete(installPath + "\\ViewsUninstall.exe");
				Directory.Delete(installPath);
			}
		}
		catch (Exception) { }
	}
}
