using System;
using System.IO;
using System.Diagnostics;
using StringBuilder = System.Text.StringBuilder;

class ViewsUninstall
{
	static void Main(string[] args)
	{
		new ViewsUninstall().Go(args);
	}

	void Go(string[] args)
	{
		string gacutil_exe = null, csc_rsp = null, version = null;
		bool success = true;

		foreach (string s in args)
		{
			if (s.IndexOf("gacutil=") == 0)
			{
				gacutil_exe = s.Substring(s.IndexOf("=")+1);
			}

			if (s.IndexOf("version=") == 0)
			{
				version = s.Substring(s.IndexOf("=")+1);
			}

			if (s.IndexOf("cscrsp=") == 0) 
			{
				csc_rsp = s.Substring(s.IndexOf("=")+1);
			}
		}

		if (args.Length != 3 || gacutil_exe == null
				|| version == null || csc_rsp == null)
		{
			Console.WriteLine("Proper usage: ViewsUnisntall.exe gacutil={x:\\path\\gacutil.exe} version={x.x.x.x} cscrsp={x:\\path\\csc.rsp}\n(use \" marks around values if they include spaces)");
			Console.WriteLine("There should be an uninstall.bat file in the installed directory with the corret arguments, try running that instead of the ViewsUninstall.exe.");
			throw new Exception("Arguments missing or malformed.");
		}

		Console.WriteLine("\n\n");

		try
		{
			ProcessStartInfo psi = new ProcessStartInfo();
			psi.FileName = gacutil_exe;
			psi.Arguments = "/u \"Views, version=" + version + "\"";
			psi.RedirectStandardOutput = true;
			psi.UseShellExecute = false;

			Process.Start(psi);
			Console.WriteLine("DLL removed from the GAC.");
		}
		catch (Exception)
		{
			Console.WriteLine("Error remove Views.dll from the GAC, to remove Views from the GAC run gacutil /u \"Views, version ...\" (some partial matching string for the string displayed by gacutil /l).");
			success = false;
		}

		Console.WriteLine("\n\n");

		try
		{
			StreamReader sr = new StreamReader(csc_rsp);
			string[] lines = sr.ReadToEnd().Split('\n');
			sr.Close();

			StreamWriter sw = new StreamWriter(csc_rsp);
			for(int i = 0; i < lines.Length; i++)
			{
				if (i != 0) sw.WriteLine();

				if (lines[i].ToLower().IndexOf(Directory.GetCurrentDirectory().ToLower()) == -1)
					sw.Write(lines[i].Trim());
			}
			sw.Close();
			Console.WriteLine("DLL reference removed from the compiler.");
		}
		catch (Exception)
		{
			Console.WriteLine("Error removing the view reference from the csc.rsp file, you should manually edit this file and remove the reference.  csc.rsp should be in the same directory as csc.exe");
			success = false;
		}

		Console.WriteLine("\n\n");

		try
		{
			File.Delete("Views.dll");
			File.Delete("Views.netmodule");
			File.Delete("Views.snk");
			Console.WriteLine("Views files deleted.");
		}
		catch (Exception)
		{
			Console.WriteLine("Error deleting one or more of the files Views.dll, Views.netmodule, Views.snk");
			success = false;
		}

		Console.WriteLine("\n\n");
		if (success) Console.WriteLine("DLL unregistered successfully.");

	}

}
